﻿--No.1----------------------------创建表、添加数据-------------------------
---------------------------------------------------------------------------
create table Region
(
Id int identity primary key,
Name nvarchar(20),
ParentId int 
)
go
insert into Region(Name,ParentId) values('广东',NULL)
insert into Region(Name,ParentId) values('深圳',1)
insert into Region(Name,ParentId) values('惠州',1)
insert into Region(Name,ParentId) values('罗湖区',2)
insert into Region(Name,ParentId) values('福田区',2)
insert into Region(Name,ParentId) values('龙岗区',2)
insert into Region(Name,ParentId) values('惠阳区',3)
insert into Region(Name,ParentId) values('龙门县',3)
insert into Region(Name,ParentId) values('华强北',5)
insert into Region(Name,ParentId) values('体育馆',5)

select * from Region

--No.2------------------------正向递归-------------------------------------
---------------------------------------------------------------------------

/*
 * summary:递归获取所有子节点
*/
create function GetRecursiveChildren
(
@Id int
)
returns @t table(Id int,ParentId int,Name nvarchar(20), [Level] int)
begin
	declare @i int
	set @i = 1
	--根节点，Level = 0
	insert into @t select @Id,@id,(select Name from Region where Id = @id),0
	--直属子节点，Level = 1
	insert into @t select Id,ParentId,Name,@i from Region where ParentId = @Id
	
	--如果没有新的值插入，循环结束
	while @@rowcount<>0
	begin
		set @i = @i + 1;
		insert into @t
		select 
			a.Id,a.ParentId,a.Name,@i
		from
			Region a, @t b
		where
			a.ParentId = b.Id and b.Level = @i - 1
	end
	return
end
go
--调用函数
select * from GetRecursiveChildren(2)


--No.3-----------------------------CTE递归---------------------------------
---------------------------------------------------------------------------
declare @id int
set @id = 2
;with t as--如果CTE前面有语句，需要用分号隔断
(
select Id, ParentId, Name
from Region
where Id = @id
union all
select r1.Id,r1.ParentId,r1.Name
from Region r1 join t as r2 on r1.ParentId = r2.Id
)
select * from t order by Id

--No.4------------------------逆向递归-------------------------------------
---------------------------------------------------------------------------

/*
 * summary:递归获取父节点
*/
create function GetRecursiveParent
(
@Id int
)
returns @t table(Id int,ParentId int,Name nvarchar(20), [Level] int)
as
begin
	declare @i int
	set @i = 1
	--插入末节点，Level = 0
	insert into @t select @Id,@id,(select Name from Region where Id = @id),0
	--插入末节点的父节点，Level = 1
	insert into @t select Id,ParentId,Name,@i from Region 
	where Id = (select ParentId from Region where Id = @Id)
	--如果没有新的值插入，循环结束
	while @@rowcount<>0
	begin
		set @i = @i + 1;
		insert into @t
		select 
			a.Id,a.ParentId,a.Name,@i
		from
			Region a, @t b
		where
			a.Id = b.ParentId and b.Level = @i - 1
	end
	return	
end
go
--调用函数
select * from GetRecursiveParent(10)
go


--No.5------------------------面包屑---------------------------------------
---------------------------------------------------------------------------
/*
 * summary:递归获取面包屑
*/
create function GetLevel
(
@Id int
)
returns @level table(IdLevel varchar(100),NameLevel nvarchar(200))
as
begin
	declare @IdLevel varchar(100),@NameLevel nvarchar(200),@Name nvarchar(50)
	select @IdLevel = cast(@Id as varchar(10))
	select @NameLevel = (select Name from Region where Id = @Id)
	
	while(exists(select Id,ParentId from Region where Id = (select ParentId from Region where Id = @Id)))
	begin
		select @Id = Id,@Name = Name from Region where Id = (select ParentId from Region where Id = @Id)
		select @IdLevel = cast(@Id as varchar(10)) + '>' + @IdLevel
		select @NameLevel = @Name + '>' + @NameLevel
	end
	insert into @level select @IdLevel,@NameLevel
	return
end
go
--调用函数
select * from GetLevel(10)
go




